@@ -1,13 +1,11 @@
 #!/usr/bin/python
-from io import BytesIO
 import re
 import struct
 import logging
-import lief
-import sys
-from collections import OrderedDict
+from io import BytesIO
 
 from smda.common.labelprovider.GoLabelProvider import GoSymbolProvider
+from smda.common.labelprovider.DelphiKbSymbolProvider import DelphiKbSymbolProvider
 
 LOGGER = logging.getLogger(__name__)
 
@@ -17,6 +15,7 @@ class LanguageAnalyzer(object):
     def __init__(self, disassembly):
         self.disassembly = disassembly
         self.go_resolver = GoSymbolProvider(None)
+        self.delphi_kb_resolver = DelphiKbSymbolProvider(None)
         self.strings = None
 
     def validPEHeader(self):
@@ -99,52 +98,109 @@ class LanguageAnalyzer(object):
     def checkGo(self):
         return self.getGoScore() > 0.5
 
-    def getDelphiObjects(self):
-        binary = self.disassembly.binary_info.binary
-        base_addr = self.disassembly.binary_info.base_addr
-        #Find T-String-Constructs
-        #(T-Strings with function-addresses before them)
-        #(T-Strings start with a capital T, followed by any capital letter)
-        t_objects = {}
-        for match in re.finditer(b"(?P<length>.)(?P<t_string>T[A-Z][a-zA-Z0-9]{4,128})", binary):
-            if not len(match.group("t_string")) - ord(match.group("length")) in [-3, -2, -1, 0, 1, 2, 3]:  # ord(match.group("length")) != len(match.group("t_string")):
-                continue
-            t_object_name = match.group("t_string")[:ord(match.group("length"))]
-            addresses = []
-            t_string_pos = base_addr + match.span()[1] - (len(match.group("t_string")) + len(match.group("length")))
-            num_addresses = 0
-            address_offset = match.start()
-            found_string_ref = False
-            # filter address arrays with form <reference>?<len_t_string><t_string>
-            if t_string_pos == (struct.unpack("<I", binary[address_offset - 5:address_offset - 1])[0] + 1):
-                continue
+    def parseDelphiString(self, buffer):
+        parsed_string = ""
+        if len(buffer) > 0:
+            length = buffer[0]
+            try:
+                parsed_string = buffer[1:1 + length].decode()
+            except:
+                parsed_string = "<invalid>"
+        return parsed_string
 
-            while num_addresses < 1000 and address_offset > 4:
-                num_addresses += 1
-                address_offset -= 4
-                address = struct.unpack("<I", binary[address_offset:address_offset + 4])[0]
-                if address == t_string_pos:
-                    found_string_ref = True
-                    LOGGER.debug("object end marker found " + str(t_object_name) + " 0x%08x " % (t_string_pos - base_addr) + "0x%08x" % (t_string_pos))
-                    t_objects[t_object_name] = []
-                    for address in addresses:
-                        if self.disassembly.isAddrWithinMemoryImage(address):
-                            t_objects[t_object_name].append(address)
-                        else:
-                            LOGGER.debug("outside range: 0x%08x", address)
-                    break
-                #if the first address is invalid, break
-                elif num_addresses == 1 and not  self.disassembly.isAddrWithinMemoryImage(address):
-                    break
-                addresses.append(address)
-            if not found_string_ref:
-                LOGGER.debug("no object end marker found" + str(t_object_name) + "0x%08x" % (t_string_pos - base_addr) + "0x%08x" % (t_string_pos))
-        return t_objects
+    def getDelphiObjects(self):
+        image_base = self.disassembly.binary_info.base_addr
+        data = BytesIO(self.disassembly.binary_info.binary)
+        function_offsets = set()
+        name_mapping = {}
+        while data.read(4) != b'':
+            data.seek(data.tell()-4)
+            offset = data.tell()
+            potential_vmt_self_ptr = int.from_bytes(data.read(4), byteorder="little")
+            saved_scan_offset = data.tell()
+            # Delphi VMTs have pointers that indicate their start, spanning an "address frame" that can be searched for
+            if offset + image_base + int('4C', base=16) == potential_vmt_self_ptr:
+                data.seek(saved_scan_offset)
+                interface_table = int.from_bytes(data.read(4), byteorder="little")
+                auto_table = int.from_bytes(data.read(4), byteorder="little")
+                init_table = int.from_bytes(data.read(4), byteorder="little")
+                type_info = int.from_bytes(data.read(4), byteorder="little")
+                field_table = int.from_bytes(data.read(4), byteorder="little")
+                method_table = int.from_bytes(data.read(4), byteorder="little")
+                dynamic_table = int.from_bytes(data.read(4), byteorder="little")
+                class_name = int.from_bytes(data.read(4), byteorder="little")
+                instance_size = int.from_bytes(data.read(4), byteorder="little")
+                class_name_saved_offset = data.tell()
+                # not used currently but a candidate for future provided meta data
+                class_name_string = self.parseDelphiString(self.disassembly.binary_info.binary[class_name - image_base:])
+                data.seek(class_name_saved_offset)
+                data.read(8)
+                # search for addresses within our known structures
+                first_address = int.from_bytes(data.read(4), byteorder="little")
+                if self.disassembly.binary_info.isInCodeAreas(first_address):
+                    data.seek(data.tell() - 4)
+                    for _ in range(8):
+                        function_offsets.add(int.from_bytes(data.read(4), byteorder="little"))
+                    if dynamic_table > 0:
+                        data.seek(dynamic_table - image_base)
+                        table_length = int.from_bytes(data.read(2), byteorder="little")
+                        data.read(2 * table_length)
+                        for _ in range(table_length):
+                            function_offsets.add(int.from_bytes(data.read(4), byteorder="little"))
+                        data.seek(potential_vmt_self_ptr - image_base)
+                        while data.tell() < dynamic_table - image_base:
+                            function_offsets.add(int.from_bytes(data.read(4), byteorder="little"))
+                    else:
+                        data.seek(potential_vmt_self_ptr - image_base)
+                        while data.tell() < class_name - image_base:
+                            function_offsets.add(int.from_bytes(data.read(4), byteorder="little"))
+                    if method_table > 0:
+                        # this should at least work for Delphi 6
+                        data.seek(method_table - image_base)
+                        length = int.from_bytes(data.read(2), byteorder="little")
+                        for _ in range(length):
+                            length_entry = int.from_bytes(data.read(2), byteorder="little")
+                            method_offset = int.from_bytes(data.read(4), byteorder="little")
+                            method_name = self.parseDelphiString(self.disassembly.binary_info.binary[data.tell():])
+                            name_mapping[method_offset] = method_name
+                            function_offsets.add(method_offset)
+                            data.seek(data.tell() + length_entry - 6)
+                        data.seek(saved_scan_offset)
+                    if interface_table > 0:
+                        data.seek(interface_table - image_base)
+                        data.read(20)
+                        start_interface = int.from_bytes(data.read(4), byteorder="little")
+                        data.seek(start_interface - image_base)
+                        bytes_read = int.from_bytes(data.read(4), byteorder="little")
+                        while self.disassembly.binary_info.isInCodeAreas(bytes_read):
+                            function_offsets.add(bytes_read)
+                            bytes_read = int.from_bytes(data.read(4), byteorder="little")
+                        data.seek(saved_scan_offset)
+                else:
+                    data.seek(saved_scan_offset)
+            else:
+                data.seek(saved_scan_offset)
+        functions = {}
+        for offset in function_offsets:
+            if self.disassembly.binary_info.isInCodeAreas(offset):
+                function_name = name_mapping[offset]if offset in name_mapping else ""
+                functions[offset] = function_name
+        return functions
 
     def getGoObjects(self):
         self.go_resolver.update(self.disassembly.binary_info)
         return self.go_resolver.getFunctionSymbols()
 
+    def getDelphiKbScore(self):
+        return 1.0 if self.disassembly.binary_info.binary.startswith(b"IDR Knowledge Base File") else 0.0
+
+    def checkDelphiKb(self):
+        return self.getDelphiKbScore() == 1
+
+    def getDelphiKbObjects(self):
+        self.delphi_kb_resolver.update(self.disassembly.binary_info)
+        return self.delphi_kb_resolver.getFunctionSymbols()
+
     def identify(self):
         result = {
             #programming language : probability
@@ -161,6 +217,7 @@ class LanguageAnalyzer(object):
             result["_count_delphi_objects"] = len(t_objects)
             if len(t_objects) > 5 and functions > 10:
                 result["delphi"] = max(result.get("delphi", 0), 0.9)
+        result["delphi_kb_file"] = self.getDelphiKbScore()
         #.NET
         result[".net"] = self.getDotNetScore()
         #VISUALBASIC
